Class: Commander::UI::ProgressBar

Inherits:
Object
  • Object
show all
Defined in:
lib/commander/user_interaction.rb

Overview

Progress Bar

Terminal progress bar utility. In its most basic form requires that the developer specifies when the bar should be incremented. Note that a hash of tokens may be passed to #increment, (or returned when using Object#progress).

uris = %w( 
  http://vision-media.ca
  http://yahoo.com
  http://google.com
  )

bar = Commander::UI::ProgressBar.new uris.length, options
threads = []
uris.each do |uri|
  threads << Thread.new do
    begin
      res = open uri
      bar.increment :uri => uri
    rescue Exception => e
      bar.increment :uri => "#{uri} failed"
    end
  end
end
threads.each { |t| t.join }

The Object method #progress is also available:

progress uris, :width => 10 do |uri|
  res = open uri
  { :uri => uri } # Can now use :uri within :format option
end

Instance Method Summary collapse

Constructor Details

#initialize(total, options = {}) ⇒ ProgressBar

Creates a new progress bar.

Options

:title              Title, defaults to "Progress"
:width              Width of :progress_bar
:progress_str       Progress string, defaults to "="
:incomplete_str     Incomplete bar string, defaults to '.'
:format             Defaults to ":title |:progress_bar| :percent_complete% complete "
:tokens             Additional tokens replaced within the format string
:complete_message   Defaults to "Process complete"

Tokens

:title 
:percent_complete
:progress_bar
:step
:steps_remaining
:total_steps
:time_elapsed
:time_remaining


313
314
315
316
317
318
319
320
321
322
# File 'lib/commander/user_interaction.rb', line 313

def initialize total, options = {}
  @total_steps, @step, @start_time = total, 0, Time.now
  @title = options.fetch :title, 'Progress'
  @width = options.fetch :width, 25
  @progress_str = options.fetch :progress_str, '='
  @incomplete_str = options.fetch :incomplete_str, '.'
  @complete_message = options.fetch :complete_message, 'Process complete'
  @format = options.fetch :format, ':title |:progress_bar| :percent_complete% complete '
  @tokens = options.fetch :tokens, {}
end

Instance Method Details

#completed?Boolean

Whether or not the operation has completed.

Returns:

  • (Boolean)


400
401
402
# File 'lib/commander/user_interaction.rb', line 400

def completed?
  @step == @total_steps
end

#erase_lineObject

Erase previous terminal line.



417
418
419
420
# File 'lib/commander/user_interaction.rb', line 417

def erase_line
  # highline does not expose the output stream
  $terminal.instance_variable_get('@output').print "\r\e[K"
end

#finished?Boolean

Whether or not the operation is complete, and we have finished.

Returns:

  • (Boolean)


393
394
395
# File 'lib/commander/user_interaction.rb', line 393

def finished?
  @step == @total_steps + 1
end

#generate_tokensObject

Generates tokens for this step.



362
363
364
365
366
367
368
369
370
371
372
373
374
# File 'lib/commander/user_interaction.rb', line 362

def generate_tokens
  {
    :title => @title,
    :percent_complete => percent_complete,
    :progress_bar => progress_bar, 
    :step => @step,
    :steps_remaining => steps_remaining,
    :total_steps => @total_steps, 
    :time_elapsed => "%0.2fs" % time_elapsed,
    :time_remaining => "%0.2fs" % time_remaining,
  }.
  merge! @tokens
end

#increment(tokens = {}) ⇒ Object

Increment progress. Optionally pass tokens which can be displayed in the output format.



408
409
410
411
412
# File 'lib/commander/user_interaction.rb', line 408

def increment tokens = {}
  @step += 1
  @tokens.merge! tokens if tokens.is_a? Hash
  show
end

#percent_completeObject

Completion percentage.



327
328
329
# File 'lib/commander/user_interaction.rb', line 327

def percent_complete
  @step * 100 / @total_steps
end

#progress_barObject

Formatted progress bar.



355
356
357
# File 'lib/commander/user_interaction.rb', line 355

def progress_bar
  (@progress_str * (@width * percent_complete / 100)).ljust @width, @incomplete_str
end

#showObject

Output the progress bar.



379
380
381
382
383
384
385
386
387
388
# File 'lib/commander/user_interaction.rb', line 379

def show
  unless finished?
    erase_line
    if completed?
      $terminal.say UI.replace_tokens(@complete_message, generate_tokens) if @complete_message.is_a? String
    else
      $terminal.say UI.replace_tokens(@format, generate_tokens) << ' '
    end
  end
end

#steps_remainingObject

Number of steps left.



348
349
350
# File 'lib/commander/user_interaction.rb', line 348

def steps_remaining
  @total_steps - @step
end

#time_elapsedObject

Time that has elapsed since the operation started.



334
335
336
# File 'lib/commander/user_interaction.rb', line 334

def time_elapsed
  Time.now - @start_time
end

#time_remainingObject

Estimated time remaining.



341
342
343
# File 'lib/commander/user_interaction.rb', line 341

def time_remaining
  (time_elapsed / @step) * steps_remaining
end